'''
# boing: Main module
# 
# Copyright 2015, Tony Rein
# Licensed under MIT.
# '''
# 
# 
# def main():
#     '''
#     Main function of the boilerplate code is the entry point of the 'boing' executable script (defined in setup.py).
#     
#     Use doctests, those are very helpful.
#     
#     >>> main()
#     Hello
#     >>> 2 + 2
#     4
#     '''
#     
#     print("Hello")

